﻿#include "syslogconsole.h"
#include <QCheckBox>
#include <QToolBar>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tfileio.h>
#include <tbaseutil/tlogger.h>
#include <topcore/topcore.h>
#include <twidget/taction.h>
#include <twidget/tmenu.h>
#include <twidget/tsearchentry.h>
#include <twidget/ttablemodel.h>
#include <twidget/ttableview.h>
#include <twidget/tuiloaderdialog.h>
#include <twidget/tvboxlayout.h>

SysLogConsole::SysLogConsole(const QString &iModuleNameStr,
                             const QVariantMap iUrlPars,
                             QWidget *iParent) :
    TopClassAbs(iParent),
    mSearchEntry(nullptr),
    mToolbar(nullptr),
    mFilter(nullptr),
    mErrorAction(nullptr)
{
    this->appendLanguage("sys-log-console");
    this->setLicenseKey("");
    this->initModule(iModuleNameStr, iUrlPars);

    mOldDebugMode = APP->debugMode();
    initTableView();

    setCentralWidget(mLogTableView);

    connect(mLogTableView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(logTableDoubleClick(QModelIndex)));

    connect(TLogger::instance(), SIGNAL(logWrited(QString,QString,QString,QString,QString)), this, SLOT(appendLog(QString,QString,QString,QString,QString)));

    //初始化工具栏
    if (uim()) {
        //工具栏
        mToolbar = qobject_cast<QToolBar*>(uim()->getWidget("MAIN_TOOLBAR"));
        if (mToolbar != nullptr){
            mToolbar->setWindowTitle(ttr("ToolBar"));
            mToolbar->setStyleSheet(".Q{}"); //让styleSheet生效
            mToolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
            mToolbar->setMovable(false);
            addToolBar(mToolbar);
        }

        mFilter = qobject_cast<TAction *>(uim()->getWidget("MainToolBar/filter"));

        //搜索栏
        mSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MAIN_TOOLBAR/SEARCH_ENTRY"));
    }

    if (mSearchEntry != nullptr) {
        mSearchEntry->setOptionList(TDataParse::headerItem2searchList(mLogTableView->headerItem()));
        mSearchEntry->setPlaceholderText(ttr("Search %1"));
        connect(mSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(onDoSearch(QString,QVariant)));
        this->setFocusProxy(mSearchEntry);
    }

    mFilterMenu = new TMenu(this);
    mFilterLayout = new TVBoxLayout();
    mFilterMenu->hide();

    mErrorCheck = new QCheckBox(this);
    mWarningCheck = new QCheckBox(this);
    mInfoCheck = new QCheckBox(this);
    mSqlCheck = new QCheckBox(this);
    mJSCheck = new QCheckBox(this);
    mOtherCheck = new QCheckBox(this);
    mWebDebugCheck = new QCheckBox(this);

    mErrorCheck->setText(ttr("ERROR"));
    mWarningCheck->setText(ttr("WARN"));
    mInfoCheck->setText(ttr("INFO"));
    mSqlCheck->setText(ttr("SQL"));
    mJSCheck->setText(ttr("JSPRINT"));
    mOtherCheck->setText(ttr("Other"));
    mWebDebugCheck->setText(ttr("WebDebug"));

    mErrorCheck->setChecked(true);
    mWarningCheck->setChecked(true);
    mInfoCheck->setChecked(true);
    mSqlCheck->setChecked(true);
    mJSCheck->setChecked(true);
    mOtherCheck->setChecked(true);

    if(APP->debugMode() > 1) {
        mWebDebugCheck->setChecked(true);
    } else {
        mWebDebugCheck->setChecked(false);
    }

    connect(mErrorCheck, SIGNAL(clicked(bool)), this, SLOT(setErrorState(bool)));
    connect(mWarningCheck, SIGNAL(clicked(bool)), this, SLOT(setWarningState(bool)));
    connect(mInfoCheck, SIGNAL(clicked(bool)), this, SLOT(setInfoState(bool)));
    connect(mSqlCheck, SIGNAL(clicked(bool)), this, SLOT(setSqlState(bool)));
    connect(mJSCheck, SIGNAL(clicked(bool)), this, SLOT(setJsState(bool)));
    connect(mOtherCheck, SIGNAL(clicked(bool)),  this,SLOT(setOtherState(bool)));
    connect(mWebDebugCheck, SIGNAL(clicked(bool)), this, SLOT(setHttpDebug(bool)));

    mFilterLayout->addWidget(mErrorCheck);
    mFilterLayout->addWidget(mWarningCheck);
    mFilterLayout->addWidget(mInfoCheck);
    mFilterLayout->addWidget(mSqlCheck);
    mFilterLayout->addWidget(mJSCheck);
    mFilterLayout->addWidget(mOtherCheck);
    mFilterLayout->setMargin(2);

    mFilterMenu->setLayout(mFilterLayout);

//    mToolbar->insertWidget(mFilter,mWebDebugCheck);
    mWebDebugCheck->hide();

    initHistoryLog();

    refreshActionState();
    this->restoreSizeState();
}

SysLogConsole::~SysLogConsole()
{
    this->saveSizeState();
}

void SysLogConsole::appendLog(const QString &iTextStr, const QString &iTypeStr, const QString &iDateTimeStr, const QString &iSrcPosStr, const QString &iDebugLevelStr)
{
    Q_UNUSED(iDebugLevelStr);

    if (mLogTableView == nullptr) {
        return;
    }

    QVariantMap logmap;
    logmap.insert("text", iTextStr);
    logmap.insert("type", iTypeStr);
    logmap.insert("datetime", iDateTimeStr);
    logmap.insert("srcpos", iSrcPosStr);

    if(iTypeStr == "ERROR") {
        logmap.insert("typeicon", "cancel.red");
    } else if (iTypeStr == "WARN") {
        logmap.insert("typeicon", "warning.#c2bd27");
    } else if (iTypeStr == "INFO") {
        logmap.insert("typeicon", "info.blue");
    } else if (iTypeStr == "SQL") {
        logmap.insert("typeicon", "database.green");
    } else if (iTypeStr == "JSPRINT") {
        logmap.insert("typeicon", "cube.#267ca1");
    } else {
        logmap.insert("typeicon", "gear.black");
    }
    mLogTableView->appendRow(logmap);
    //mLogTableView->resizeRowsToContents();
    //mLogTableView->resizeColumnsToContents();
    refresh();
    mLogTableView->setCurrentIndex(mLogTableModel->index(mLogTableModel->rowCount() - 1, 0));
}

void SysLogConsole::clearTable()
{
    if (mLogTableView == nullptr) {
        return;
    }

    mLogTableView->clearData();
}

void SysLogConsole::filter()
{
    if (mFilterMenu == nullptr) {
        return;
    }

    QPoint mousePos = QCursor::pos();
    mFilterMenu->move(mousePos);
    mFilterMenu->show();
}

void SysLogConsole::onDoSearch(QString iKeyStr, QVariant iFilterVar)
{
    refresh();

    QStringList filterLst;
    for (QVariant filter : iFilterVar.toList()) {
        filterLst.append(filter.toString());
    }

    QModelIndex index;
    QVariantMap indexMap;
    for (int row = 0; row < mLogTableModel->rowCount(); ++row) {
        if (mLogTableView->isRowHidden(row)) {
            continue;
        }

        index = mLogTableModel->index(row, 0);
        indexMap = mLogTableModel->rowDataMap(index);

        //全字段过滤
        if (filterLst.count() == 0) {
            for (QVariantMap::Iterator iter = indexMap.begin(); iter != indexMap.end(); ++iter) {
                if (QVariant(iter.value()).toString().toUpper().contains(iKeyStr.toUpper())) {
                    mLogTableView->showRow(row);
                    break;
                } else {
                   mLogTableView->hideRow(row);
                }
            }
        } else {
            for (QString filterStr : filterLst) {
                if (indexMap.value(filterStr).toString().toUpper().contains(iKeyStr.toUpper())) {
                    mLogTableView->showRow(row);
                    break;
                } else {
                    mLogTableView->hideRow(row);
                }
            }
        }
    }
}

void SysLogConsole::setDebugLevel(int iDebugLevel)
{
    TLOGGER->setDebugLevel(iDebugLevel);
}

int SysLogConsole::debugLevel() const
{
    return TLOGGER->debugLevel();
}

void SysLogConsole::refresh()
{
    setErrorState(mErrorCheck->isChecked());
    setWarningState(mWarningCheck->isChecked());
    setInfoState(mInfoCheck->isChecked());
    setSqlState(mSqlCheck->isChecked());
    setJsState(mJSCheck->isChecked());
    setOtherState(mOtherCheck->isChecked());
    alertOk(ttr("Data loaded"));
}

void SysLogConsole::setErrorState(bool iBol)
{
    QModelIndex index;
    QVariantMap indexMap;
    for (int row = 0; row < mLogTableModel->rowCount(); ++row) {
        index = mLogTableModel->index(row, 0);
        indexMap = mLogTableModel->rowDataMap(index);

        if (indexMap.value("type").toString().toUpper() == QString("ERROR")) {
            if (iBol == false) {
                mLogTableView->hideRow(row);
            } else {
                mLogTableView->showRow(row);
            }
        }
    }
}

void SysLogConsole::setWarningState(bool iBol)
{
    QModelIndex index;
    QVariantMap indexMap;
    for (int row = 0; row < mLogTableModel->rowCount(); ++row) {
        index = mLogTableModel->index(row, 0);
        indexMap = mLogTableModel->rowDataMap(index);

        if (indexMap.value("type").toString().toUpper() == QString("WARN")) {
            if(iBol == false) {
                mLogTableView->hideRow(row);
            } else {
                mLogTableView->showRow(row);
            }
        }
    }
}

void SysLogConsole::setInfoState(bool iBol)
{
    QModelIndex index;
    QVariantMap indexMap;
    for (int row = 0; row < mLogTableModel->rowCount(); ++row) {
        index = mLogTableModel->index(row, 0);
        indexMap = mLogTableModel->rowDataMap(index);

        if (indexMap.value("type").toString().toUpper() == QString("INFO")) {
            if (iBol == false) {
                mLogTableView->hideRow(row);
            } else {
                mLogTableView->showRow(row);
            }
        }
    }
}

void SysLogConsole::setSqlState(bool iBol)
{
    QModelIndex index;
    QVariantMap indexMap;
    for (int row = 0; row < mLogTableModel->rowCount(); ++row) {
        index = mLogTableModel->index(row, 0);
        indexMap = mLogTableModel->rowDataMap(index);

        if (indexMap.value("type").toString().toUpper() == QString("SQL")) {
            if (iBol == false) {
                mLogTableView->hideRow(row);
            } else {
                mLogTableView->showRow(row);
            }
        }
    }
}

void SysLogConsole::setJsState(bool iBol)
{
    QModelIndex index;
    QVariantMap indexMap;
    for (int row = 0; row < mLogTableModel->rowCount(); ++row) {
        index = mLogTableModel->index(row, 0);
        indexMap = mLogTableModel->rowDataMap(index);

        if (indexMap.value("type").toString().toUpper() == QString("JSPRINT")) {
            if (iBol == false) {
                mLogTableView->hideRow(row);
            } else {
                mLogTableView->showRow(row);
            }
        }
    }
}

void SysLogConsole::setOtherState(bool iBol)
{
    QModelIndex index;
    QVariantMap indexmap;
    for (int row = 0; row < mLogTableModel->rowCount(); ++row) {
        index = mLogTableModel->index(row, 0);
        indexmap = mLogTableModel->rowDataMap(index);

        QString type = indexmap.value("type").toString().toUpper();
        if ((type != QString("ERROR")) && (type != QString("WARN")) && (type != QString("INFO"))
                && (type != QString("SQL")) && (type != QString("JSPRINT"))) {
            if (iBol == false) {
                mLogTableView->hideRow(row);
            } else {
                mLogTableView->showRow(row);
            }
        }
    }
}

void SysLogConsole::setHttpDebug(bool iBol)
{
    APP->setDebugMode(iBol ? 2 : mOldDebugMode);
    if (APP->debugMode() > 1) {
        mWebDebugCheck->setChecked(true);
    } else {
        mWebDebugCheck->setChecked(false);
    }
}

void SysLogConsole::logTableDoubleClick(QModelIndex iModelIndex)
{
    QVariantMap detailMap = mLogTableModel->rowDataMap(iModelIndex);
    TUiloaderDialog dialog;
    dialog.setTitle(ttr("Detail Info"));
    dialog.setSelf(this);
    dialog.setScriptEngine(APP->scriptEngine());
    dialog.resize(600, 300);
    dialog.setUiStr(ui("logconsoledetail").toString());
    dialog.setButtons(QStringList() << ttr("Ok") + ":Ok:Ok:Primary");
    dialog.loadValues(detailMap);
    dialog.run(false);
}

void SysLogConsole::initTableView()
{
    mLogTableView = new TTableView(this);
    mLogTableView->setShowGrid(false);
    mLogTableView->setSelectionMode(QAbstractItemView::ExtendedSelection);
    mLogTableView->verticalHeader()->setVisible(false);
    mTableDataKeyLst << "datetime" << "typeicon" <<"type" << "text" << "srcpos";
    mLogTableView->setDataKeyList(mTableDataKeyLst);
    mLogTableView->setPrimaryKey("datetime");
    mLogTableView->horizontalHeader()->setMinimumSectionSize(75);
    mLogTableView->horizontalHeader()->setStretchLastSection(true);

    QVariantList hItems;
    hItems << QVariantMap();
    //datetime
    hItems << TDataParse::variantList2Map(
                        QVariantList() << "name" << "datetime" \
                        << "display" << ttr("DateTime") \
                        << "dataType" << "string" \
                        << "size" << "90"\
                        << "resizeMode" << "Interactive" \
                        << "displayRole" << "$datetime" \
                        << "search" << "string"
                        );

    //type
    hItems << TDataParse::variantList2Map(
                        QVariantList() << "name" << "type" \
                        << "display" << ttr("Type") \
                        << "dataType" << "string" \
                        << "size" << "100"\
                        << "resizeMode" << "Interactive" \
                        << "displayRole" << "$type" \
                        << "decorationRole" << "$typeicon" \
                        << "search" << "string"
                        );

    //text
    hItems << TDataParse::variantList2Map(
                        QVariantList() << "name" << "text" \
                        << "display" << ttr("Text") \
                        << "dataType" << "string" \
                        << "size" << "800"\
                        << "resizeMode" << "Interactive" \
                        << "displayRole" << "$text" \
                        << "search" << "string"
                        );

    //srcpos
    hItems << TDataParse::variantList2Map(
                        QVariantList() << "name" << "srcpos" \
                        << "display" << ttr("SrcPos") \
                        << "dataType" << "string" \
                        << "size" << "400"\
                        << "resizeMode" << "Interactive" \
                        << "displayRole" << "$srcpos" \
                        << "search" << "string"
                        );

    mLogTableView->setHeaderItem(hItems);

    mLogTableModel = qobject_cast<TTableModel *>(mLogTableView->model());
}

void SysLogConsole::initHistoryLog()
{
    for (QVariant row : TLogger::instance()->historyLogs()) {
        QStringList rowLst = row.toStringList();
        appendLog(rowLst.value(0), rowLst.value(1), rowLst.value(2), rowLst.value(3), "");
    }
}
