﻿#include "topwin8desktop.h"
#include <QWidget>
#include <QWheelEvent>
#include <QScrollBar>
#include <QMimeData>
#include <QToolBar>
#include <QFileDialog>
#include <QGuiApplication>
#include <QImageReader>
#include <QPainter>
#include <QToolTip>
#include <QRgb>
#include <tbaseutil/ttheme.h>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tresource.h>
#include <tbaseutil/tfileio.h>
#include <tbaseutil/tdataresponse.h>
#include <twidget/tscrollarea.h>
#include <twidget/tsearchentry.h>
#include <twidget/tuiloaderdialog.h>
#include <twidget/tuiloader.h>
#include <twidget/tmessagebox.h>
#include <twidget/tcombobox.h>
#include <twidget/ttableviewdialog.h>
#include <twidget/tmessagebar.h>
#include <twidget/tlineedit.h>
#include <twidget/tgridlayout.h>
#include <topcore/topuim.h>
#include <topcore/topcore.h>
#include <topcore/topclassabs.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include "topwin8desktopgroup.h"
#include "topdesktopitemabs.h"

const static QString cWin8DesktopClassNameStr = "win8desktop";

enum WndSize
{
    WndSize_CellWidth   = 95,
    WndSize_CellHeight  = 95,
    WndSize_WndWidth    = 800,
    WndSize_WndHeight   = 600,
    WndSize_DelayTime   = 200,
    WndSize_Spacing     = 20,
    WndSize_Margin      = 5,
    WndSize_GroupsHSpace = 5,
    WndSize_GroupsVSpace = 5,
};

enum DesktopAction
{
    DesktopAction_Refresh       = 0,
    DesktopAction_SetEditable   = 1,
    DesktopAction_SaveModified  = 2,
    DesktopAction_SetBackground = 3,
    DesktopAction_AddGroup      = 4
};

class TopWin8DesktopPrivate
{
    Q_DECLARE_PUBLIC(TopWin8Desktop)
public:
    TopWin8DesktopPrivate(TopWin8Desktop *qptr);

    TScrollArea *scrollArea;
    QWidget *scrollCenterWgt;
    TSearchEntry *searchEntry;
    QToolBar *toolBar;
    QMenu *desktopMenu = nullptr;
    QMap<DesktopAction, QAction *> desktopActions;
    TopDesktopItemAbs::State state;
    QVariantMap dataVarMap;
    QList<TopWin8DesktopGroup *> groupsLst;
    bool isDataModifiedBol;
    static QMap<QString, QString> i18NMap;

protected:
    TopWin8Desktop * const q_ptr;
};


QMap<QString, QString> TopWin8DesktopPrivate::i18NMap = {
    {"Background Color", QObject::tr("Background Color")},
    {"Background Image", QObject::tr("Background Image")},
    {"DockArea Alignment", QObject::tr("DockArea Alignment")},
    {"Horizontal", QObject::tr("Horizontal")},
    {"Vertical", QObject::tr("Vertical")},
    {"Module Click Type", QObject::tr("Module Click Type")},
    {"Double Click", QObject::tr("Double Click")},
    {"Click", QObject::tr("Click")}
};

TopWin8DesktopPrivate::TopWin8DesktopPrivate(TopWin8Desktop *qptr)
    : q_ptr(qptr)
{

}

/*****************************************/

TopWin8Desktop::TopWin8Desktop(QWidget *iParent) :
    QFrame(iParent),
    d_ptr(new TopWin8DesktopPrivate(this))
{
    Q_D(TopWin8Desktop);

    this->setMouseTracking(true);
    this->resize(WndSize_WndWidth, WndSize_WndHeight);
    this->setObjectName("DESKTOP");
    d->scrollArea = new TScrollArea(this);
    d->scrollArea->setObjectName("SCROLLAREA");
    d->scrollCenterWgt = new QWidget(d->scrollArea);
    d->scrollCenterWgt->setObjectName("CENTERWIDGET");
    d->scrollArea->setWidget(d->scrollCenterWgt);
    d->scrollArea->setWidgetResizable(true);

    QVBoxLayout *centerLayout = new QVBoxLayout(this);
    centerLayout->setSpacing(0);
    setLayout(centerLayout);

    TopUim *uim = new TopUim(this);
    {
        QVariantList items;
        items.push_back(QVariantMap{{"type", "stretcher"}});
        items.push_back(QVariantMap{{"type", "searchentry"},{"name", "SEARCH_ENTRY"}});
        uim->setUim(QVariantMap{{ "MAIN_TOOLBAR", items}});
    }

    QToolBar *toolbar = qobject_cast<QToolBar*>(uim->getWidget("MAIN_TOOLBAR"));
    if (toolbar != nullptr) {
        toolbar->setProperty("SS", "TITLE");
        toolbar->setStyleSheet("background-color:transparent;border-color:transparent;");
        toolbar->installEventFilter(this);
        toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        toolbar->setMovable(false);
    }
    centerLayout->addWidget(toolbar, 0, Qt::AlignTop);
    centerLayout->addWidget(d->scrollArea, 1);
    TSearchEntry *searchEntry = qobject_cast<TSearchEntry *>(uim->getWidget("MAIN_TOOLBAR/SEARCH_ENTRY"));
    if (searchEntry != nullptr) {
        searchEntry->setProperty("SS", "TITLE");
        searchEntry->setStyleSheet("TSearchEntry[SS=\"TITLE\"] QLineEdit{border-style: solid;border-width:1px;border-radius:2px;border-color:#AAAAAA2F;}");
        searchEntry->setPlaceholderText(tr("Search"));
        searchEntry->setSearchDelayTime(WndSize_DelayTime);
        connect(searchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(onDoSearch(QString)));
        d->searchEntry = searchEntry;
    }

    // 仅有权限的用户才创建编辑桌面相关的菜单和Action
    if (APP->hasRight("sys-edit-desktop")) {
        // 桌面Menu
        QMenu *desktopMenuFromDb = new QMenu(this);
        // 刷新
        QAction *refreshAction = new QAction(TRES->icon(""), tr("Refresh"), this);
        connect(refreshAction, SIGNAL(triggered()), this, SLOT(refresh()));
        d->desktopActions.insert(DesktopAction_Refresh, refreshAction);
        desktopMenuFromDb->addAction(refreshAction);
        // 编辑桌面
        QAction *editAction = new QAction(TRES->icon(""), tr("Edit Desktop"), this);
        connect(editAction, SIGNAL(triggered()), this, SLOT(editDesktop()));
        d->desktopActions.insert(DesktopAction_SetEditable, editAction);
        desktopMenuFromDb->addAction(editAction);
        // 保存
        QAction *saveAction = new QAction(TRES->icon(""), tr("Save Desktop"), this);
        connect(saveAction, SIGNAL(triggered()), this, SLOT(saveDesktop()));
        d->desktopActions.insert(DesktopAction_SaveModified, saveAction);
        desktopMenuFromDb->addAction(saveAction);
        // 设置背景颜色
        QAction *setBgAction = new QAction(TRES->icon(""), tr("Desktop Config"), this);
        connect(setBgAction, SIGNAL(triggered()), this, SLOT(showConfigDialog()));
        d->desktopActions.insert(DesktopAction_SetBackground, setBgAction);
        desktopMenuFromDb->addAction(setBgAction);
        // 添加分组
        QAction *addGroupAction = new QAction(TRES->icon(""), tr("Add Group"), this);
        connect(addGroupAction, SIGNAL(triggered()), this, SLOT(addGroup()));
        d->desktopActions.insert(DesktopAction_AddGroup, addGroupAction);
        desktopMenuFromDb->addAction(addGroupAction);
        d->desktopMenu = desktopMenuFromDb;
    }

    d->scrollArea->setStyleSheet(
                QString(".QWidget#CENTERWIDGET{background:transparent;}\n" \
                        "QScrollBar::corner{border:0px none;}\n" \
                        "QScrollBar:horizontal, QScrollBar:vertical{background:rgba(255, 255, 255, 0.3);margin:2px; height:14px;border-radius:5px;border:0px none;} \n" \
                        "QScrollBar::handle:horizontal, QScrollBar::handle:vertical{background:rgba{255, 255, 255, 0.3};margin:0px;border:0px none; border-radius:5px;} \n" \
                        "QScrollBar::add-line:horizontal, QScrollBar::add-line:vertical{background:transparent;width:0px;} \n" \
                        "QScrollBar::sub-line:horizontal, QScrollBar::sub-line:vertical{background:transparent;width:0px;} \n" \
                        "QScrollBar::add-page:vertical, QScrollBar::add-page:vertical{background:transparent;width:0px;}"));

    this->setAcceptDrops(true);
    this->installEventFilter(this);
    d->scrollArea->setMouseTracking(true);
    d->scrollCenterWgt->setMouseTracking(true);

    refresh();
}

TopWin8Desktop::~TopWin8Desktop()
{
}

void TopWin8Desktop::doLayout()
{
    Q_D(TopWin8Desktop);
    int deskcateHeightInt = 0;
    int deskcateWidthInt = 0;
    QString dockAreaStr;
    QString searchTextStr = d->searchEntry->text();
    if (searchTextStr.endsWith('>')) searchTextStr = "";
    QRegExp reg(QRegExp::escape(searchTextStr), Qt::CaseInsensitive);
    foreach (TopWin8DesktopGroup *category, d->groupsLst) {
        foreach (TopDesktopItemAbs *item, category->desktopItems()) {
            if (item->hasOpenRight()
                    && (searchTextStr.isEmpty()
                        || reg.indexIn(item->config("title").toString()) >= 0
                        || reg.indexIn(item->config("title_zhcn").toString()) >= 0
                        || reg.indexIn(item->config("title_zhtw").toString()) >= 0
                        || reg.indexIn(item->config("title_en").toString()) >= 0
                        || reg.indexIn(item->config("searchCode").toString()) >= 0))
            {
                item->setProperty("VISIBLE", QVariant(true));
            }
            else
            {
                item->setProperty("VISIBLE", QVariant(false));
            }
        }
        category->blockSignals(true);
        category->doLayout();
        dockAreaStr = category->dockArea();
        category->show();
        category->blockSignals(false);

        if (str2DockArea(dockAreaStr) == DockArea_Position)
        {
            category->move(category->position());
            if (category->geometry().bottom() > deskcateHeightInt) deskcateHeightInt = category->geometry().bottom();
            if (category->geometry().right() > deskcateWidthInt) deskcateWidthInt = category->geometry().right();
        }
    }
    QSize dockAreasSize = calcDesktopSize();
    if (deskcateHeightInt < dockAreasSize.height()) deskcateHeightInt = dockAreasSize.height();
    if (deskcateWidthInt < dockAreasSize.width()) deskcateWidthInt = dockAreasSize.width();
    d->scrollCenterWgt->setMinimumSize(deskcateWidthInt, deskcateHeightInt);
    if (deskcateHeightInt < this->height()) deskcateHeightInt = this->height();
    if (deskcateWidthInt < this->width()) deskcateWidthInt = this->width();
    d->scrollCenterWgt->resize(deskcateWidthInt, deskcateHeightInt);
    layoutDockAreaGroup();
}

void TopWin8Desktop::setConfigData(const QVariantMap &iConfigDataVarMap)
{
    Q_D(TopWin8Desktop);
    d->dataVarMap.clear();
    d->dataVarMap = iConfigDataVarMap;
    if (config("module_click_type").toString().trimmed().isEmpty())
    {
        setModuleClickType(TopDesktopItemAbs::ClickType_DoubleClick);
    }

    if (iConfigDataVarMap.contains("groups"))
    {
        foreach (TopWin8DesktopGroup *group, d->groupsLst) {
            group->deleteLater();
        }
        d->groupsLst.clear();
        d->groupsLst = createGroups(iConfigDataVarMap.value("groups").toList());
    }

    setBackground();
}

QVariantMap TopWin8Desktop::configData()
{
    Q_D(TopWin8Desktop);
    QVariantMap dataVarMap = d->dataVarMap;
    QVariantList groupsVarLst;
    foreach (TopWin8DesktopGroup *group, d->groupsLst) {
        groupsVarLst.push_back(group->configData());
    }
    dataVarMap.insert("groups", groupsVarLst);
    if (!dataVarMap.contains("background")) dataVarMap.insert("background", "");
    if (!dataVarMap.contains("backgroundImage")) dataVarMap.insert("backgroundImage", "");

    return dataVarMap;
}

void TopWin8Desktop::setConfig(const QString &iKeyStr, const QVariant &iValueVar)
{
    Q_D(TopWin8Desktop);

    d->dataVarMap.insert(iKeyStr, iValueVar);
}

QVariant TopWin8Desktop::config(const QString &iKeyStr) const
{
    Q_D(const TopWin8Desktop);
    return d->dataVarMap.value(iKeyStr);
}

void TopWin8Desktop::onDoSearch(const QString &iTextStr)
{
    Q_D(const TopWin8Desktop);
    if (!iTextStr.startsWith('<'))
    {
        doLayout();
    }
    else
    {
        if (iTextStr.endsWith('>') && APP->userName().toUpper() == "ADMIN")
        {
            QString urlAddresStr = iTextStr.mid(1, iTextStr.length() - 2).trimmed();
            foreach (TopWin8DesktopGroup *group, d->groupsLst) {
                foreach (TopDesktopItemAbs *item, group->desktopItems()) {
                    if (item->urlAddress().trimmed().toUpper() == urlAddresStr.toUpper())
                    {
                        emit urlRequested(urlAddresStr, item->configData());
                        break;
                    }
                }
            }
        }
    }
}

void TopWin8Desktop::onDataChanged()
{
    setDataModified(true);
}

void TopWin8Desktop::setDataModified(bool iModifedBol)
{
    Q_D(TopWin8Desktop);
    d->isDataModifiedBol = iModifedBol;
    if (iModifedBol) emit dataChanged();
}

bool TopWin8Desktop::isDataModified() const
{
    Q_D(const TopWin8Desktop);
    return d->isDataModifiedBol;
}

QSize TopWin8Desktop::sizeStr2QSize(const QString &iSizeStr)
{
    QString rowStr;
    QString colStr;
    QString sizeStr = iSizeStr.toUpper();
    if (sizeStr.contains('X'))
    {
        QStringList sizeStrLst = sizeStr.split('X');
        rowStr = sizeStrLst.at(1);
        colStr = sizeStrLst.at(0);
    }
    if (rowStr.isEmpty()) rowStr = "0";
    if (colStr.isEmpty()) colStr = "0";

    QSize size(colStr.toInt(), rowStr.toInt());
    return size;
}

QString TopWin8Desktop::rowColStr2SizeStr(const QString &iRowStr, const QString &iColStr)
{
    return QString("%1x%2").arg(iRowStr).arg(iColStr);
}

TopDesktopItemAbs::State TopWin8Desktop::state() const
{
    Q_D(const TopWin8Desktop);
    return d->state;
}

void TopWin8Desktop::setState(TopDesktopItemAbs::State iState)
{
    Q_D(TopWin8Desktop);
    d->state = iState;

    foreach (TopWin8DesktopGroup *group, d->groupsLst) {
        group->setState(iState);
    }
}

int TopWin8Desktop::colSpan() const
{
    QString rowColStr = config("size").toString().toUpper();
    if (rowColStr.contains("X"))
    {
        QStringList sizeStrLst = rowColStr.split("X");
        return sizeStrLst.at(0).toInt();
    }
    return 0;
}

int TopWin8Desktop::rowSpan() const
{
    QString rowColStr = config("size").toString().toUpper();
    if (rowColStr.contains("X"))
    {
        QStringList sizeStrLst = rowColStr.split("X");
        return sizeStrLst.at(1).toInt();
    }
    return 0;
}

void TopWin8Desktop::setModuleClickType(TopDesktopItemAbs::ClickType iClickType)
{
    Q_D(const TopWin8Desktop);
    setConfig("module_click_type", moduleClickType2Str(iClickType));

    foreach (TopWin8DesktopGroup *group, d->groupsLst) {
        group->setModuleClickType(iClickType);
    }
}

TopDesktopItemAbs::ClickType TopWin8Desktop::moduleClickType() const
{
    return str2ModuleClickType(config("module_click_type").toString());
}

QString TopWin8Desktop::selectImageDialog()
{
    QString filePath = QFileDialog::getOpenFileName(this, tr("Open File"), "", tr("Images (*.png *.jpeg *.jpg)"));
    if (filePath.isEmpty()) {
        return QString();
    }
    QString dstDirPath = QString("%1/resource/image/").arg(APP->appPlatformPath());
    QDir dstDir(dstDirPath);
    if (!dstDir.exists()) {
        dstDir.mkpath(dstDirPath);
    }
    QFileInfo fileInfo(filePath);
    QString fileName = fileInfo.fileName();
    QString dstFilePath = dstDirPath + fileName;\
    if (QFile::exists(dstFilePath)) {
        if (fileInfo.absoluteDir() == dstDir) {
            return fileName;
        }
        QFile::remove(dstFilePath);
    }
    if (!QFile::copy(filePath, dstFilePath)) {
        qDebug() << __FUNCTION__ << filePath << dstFilePath;
        TMessageBox::warning(this, tr("Copy image to resource directory failed."));
        return QString();
    }
    return fileName;
}

void TopWin8Desktop::setColorList(TComboBox *iBackgroundCombox)
{
    QString colorsStr =  "["
                         "\"transparent\","
                         "\"rgba(0, 0, 0, 0.9)\","
                         "\"rgba(255, 255, 255, 0.9)\","
                         "\"rgba(42, 105, 238, 0.9)\","
                         "\"rgba(239, 59, 5, 0.9)\","
                         "\"rgba(255, 153, 0, 0.9)\","
                         "\"rgba(27, 117, 27, 0.9)\","
                         "\"rgba(110, 9, 189, 0.9)\","
                         "\"rgba(0, 204, 204, 0.9)\","
                         "\"rgba(51, 153, 51, 0.9)\","
                         "\"rgba(250, 123, 56, 0.9)\","
                         "\"rgba(86, 97, 153, 0.9)\","
                         "\"rgba(173, 86, 94, 0.9)\","
                         "\"rgba(128,128,255,0.9)\","
                         "\"rgba(164,99,255,0.9)\","
                         "\"rgba(0,128,128,0.9)\","
                         "\"rgba(204,51,51,0.9)\","
                         "\"rgba(255, 191, 24, 0.9)\""
                         "]";
    QVariantList btnBkgColorVarLst = TDataParse::jsonStr2Variant(colorsStr).toList();
    //1. 下拉框添加颜色字符
    QVariantList colorVarLst;
    foreach(QVariant itemVar, btnBkgColorVarLst)
    {
        QString colorStr = itemVar.toString();

        QVariantMap comboVarMap;
        comboVarMap.insert("name", colorStr);
        comboVarMap.insert("text", colorStr);

        colorVarLst << comboVarMap;
    }
    iBackgroundCombox->setItemList(colorVarLst);

    //2. 设置颜色图标
    for(int n = 0; n < colorVarLst.count(); n++)
    {
        QVariantMap itemVarMap = colorVarLst.at(n).toMap();
        QString colorStr = itemVarMap.value("text").toString();

        int rInt = 0, gInt = 0, bInt = 0;
        QRegExp rx("(\\d+)\\s*\\,\\s*(\\d+)\\s*\\,\\s*(\\d+)");
        if (rx.indexIn(colorStr) != -1)
        {
            rInt = rx.cap(1).toInt();
            gInt = rx.cap(2).toInt();
            bInt = rx.cap(3).toInt();
        }

        //创建pixmap
        const int widthInt = 80;
        const int heightInt = 20;
        QPixmap pixColor(widthInt, heightInt);
        pixColor.fill(QColor(rInt, gInt, bInt, 210));

        iBackgroundCombox->setItemIcon(n, QIcon(pixColor));
        iBackgroundCombox->setIconSize(QSize(widthInt, heightInt));
    }
}

void TopWin8Desktop::setLabelAdjustSize(QLabel *label)
{
    label->adjustSize();
    label->setFixedWidth(20);
    label->setWordWrap(true);
    label->setAlignment(Qt::AlignCenter);
}

QString TopWin8Desktop::ttr(const QString &iStr)
{
    return TopWin8DesktopPrivate::i18NMap.value(iStr, iStr);
}

void TopWin8Desktop::refresh()
{
    TSqlSelectorV2 sqlSelector;
    sqlSelector.setTable("pub_conf");
    sqlSelector.setField("text_data");
    sqlSelector.addWhere("path", QString("%1/%2").arg(APP->currentProduct().toUpper()).arg(cWin8DesktopClassNameStr));

    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());

    TDataResponse dataRes;
    try {
        QVariant dataVar = sqlQuery.selectValue(sqlSelector);
        if (sqlQuery.lastError().isValid()) throw sqlQuery.lastError();

        setState(TopDesktopItemAbs::State_None);
        setConfigData(TDataParse::jsonStr2Variant(dataVar.toString()).toMap());

        doLayout();
        setDataModified(false);
        return;
    } catch (const TError &err){
        dataRes.setError(err);
    } catch(...){
        dataRes.setErrText(tr("Unknow Error!"));
    }
}

void TopWin8Desktop::editDesktop()
{
    if (state() == TopDesktopItemAbs::State_Editing) {
        setState(TopDesktopItemAbs::State_None);
    } else {
        setState(TopDesktopItemAbs::State_Editing);
    }
}

void TopWin8Desktop::saveDesktop()
{
    TSqlInserterV2 inserter;
    inserter.setTable("pub_conf");
    inserter.setUniqueField("path");
    QVariantMap data;
    data.insert("path", QString("%1/%2").arg(APP->currentProduct().toUpper()).arg(cWin8DesktopClassNameStr));
    data.insert("text_data", TDataParse::variant2JsonStr(configData()));
    inserter.setData(data);
    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.replaceRow(inserter);
    if (sqlQuery.lastError().isValid()) {
        TMessageBox::warning(this, "Save desktop config failed.", sqlQuery.lastError().text());
        return;
    }
    setState(TopDesktopItemAbs::State_None);
    setDataModified(false);
}

void TopWin8Desktop::showConfigDialog()
{
    QString scriptFileName(":/topclass/topdesktop/DesktopInfo.ui.js");
    QString uiStr = TFileIo::readFileString(scriptFileName);
    TUiloaderDialog dialog;
    dialog.setTitle(tr("Edit Config"));
    dialog.setIcon("setting2.white");
    dialog.setSelf(this);
    dialog.setScriptEngine(APP->scriptEngine());
    dialog.resize(550, 400);

    dialog.setUiStr(uiStr);
    dialog.loadValues(configData());
    QVariantMap retVarMap = dialog.run();
    dialog.close();
    if (retVarMap.isEmpty()) return;

    QStringList keyStrLst = retVarMap.keys();
    bool isDataChangedBol = false;
    bool isBackgroundChangedBol = false;
    bool isModuleClickTypeChangedBol = false;
    foreach (QString keyStr, keyStrLst) {
        if (config(keyStr).toString() != retVarMap.value(keyStr).toString())
        {
            if (keyStr == "background" || keyStr == "backgroundImage")
            {
                isBackgroundChangedBol = true;
            }
            else if (keyStr == "module_click_type")
            {
                isModuleClickTypeChangedBol = true;
            }
            else
            {
                isDataChangedBol = true;
            }


            setConfig(keyStr, retVarMap.value(keyStr));
        }
    }

    if (isBackgroundChangedBol) setBackground();
    if (isDataChangedBol) doLayout();
    if (isModuleClickTypeChangedBol) setModuleClickType(this->moduleClickType());
    setDataModified(isBackgroundChangedBol || isModuleClickTypeChangedBol || isDataChangedBol);
}

void TopWin8Desktop::addGroup()
{
    Q_D(TopWin8Desktop);
    if (d->desktopMenu == nullptr) {
        return;
    }

    TopWin8DesktopGroup *group = createGroup(QVariantMap());

    QColor color;
    QString bgStr = config("background").toString();
    int rInt = 0, gInt = 0, bInt = 0;
    QRegExp rx("(\\d+)\\s*\\,\\s*(\\d+)\\s*\\,\\s*(\\d+)");
    if (rx.indexIn(bgStr) != -1)
    {
        rInt = rx.cap(1).toInt();
        gInt = rx.cap(2).toInt();
        bInt = rx.cap(3).toInt();
        color = QColor(rInt, gInt, bInt);
    }
    else
    {
        color = QColor(bgStr);
    }

    if (!color.isValid())
    {
        color = QColor(TTHEME->value("BODY_BG").toString());
    }
    if (color.isValid())
    {
        //r g b
        int colorCodeInt = color.red() * 256 * 256 + color.green() * 256 + color.blue();
        int whiteCodeInt = 255 * 256 * 256 + 255 * 256 + 255;
        if (whiteCodeInt / 2 > colorCodeInt) group->setConfig("titleColor", "rgba(255, 255, 255, 0.9)");  // 颜色偏浅
        else group->setConfig("titleColor", "rgba(0, 0, 0, 0.9)");
    }

    if (group->showConfigDialog(true))
    {
        QPoint clickPosition = classPointToCenterWidgetPoint(d->desktopMenu->property("POSITION").toPoint());
        group->setPosition(clickPosition);
        d->groupsLst.append(group);
        this->doLayout();
    }
    else
    {
        delete group;
    }
}

void TopWin8Desktop::setBackground()
{
    const QString background = this->config("background").toString();
    const QString backgroundImage = this->config("backgroundImage").toString();
    if (background.isEmpty() && backgroundImage.isEmpty()) {
        this->setStyleSheet(QString(".TopWin8Desktop{border-image:url(:/topclass/topdesktop/bg.jpg);}"));
    } else if (background.isEmpty()) {
        this->setStyleSheet(QString(".TopWin8Desktop{border-image:url(%1);}")
                            .arg(this->config("backgroundImage").toString()));
    } else {
        this->setStyleSheet(QString(".TopWin8Desktop{background-color:%1;border-image:url(%2);}")
                            .arg(this->config("background").toString())
                            .arg(QString("%1/resource/image/%2").arg(APP->appPlatformPath()).arg(this->config("backgroundImage").toString())));
    }
}

void TopWin8Desktop::deleteGroup(TopWin8DesktopGroup *iGroup)
{
    Q_D(TopWin8Desktop);
    if (d->groupsLst.contains(iGroup) && iGroup && d->groupsLst.removeOne(iGroup))
    {
        iGroup->deleteLater();
        onDataChanged();
    }
}

QList<TopWin8DesktopGroup *> TopWin8Desktop::createGroups(const QVariantList &iGroupsVarLst)
{
    QList<TopWin8DesktopGroup *> groupsLst;
    foreach (QVariant groupVar, iGroupsVarLst) {
        groupsLst.push_back(createGroup(groupVar));
    }

    return groupsLst;
}

TopWin8DesktopGroup *TopWin8Desktop::createGroup(const QVariant &iGroupVar)
{
    Q_D(TopWin8Desktop);
    QVariantMap groupsVarMap = iGroupVar.toMap();

    TopWin8DesktopGroup *group = new TopWin8DesktopGroup(d->scrollCenterWgt);
    group->setProperty("MODULE", this->property("MODULE").toString());
    group->setState(state());
    group->setCellWidth(WndSize_CellWidth);
    group->setCellHeight(WndSize_CellHeight);
    group->setModuleClickType(this->moduleClickType());

    QString titleStr = groupsVarMap.value(QString("title_%1").arg(APP->language())).toString();
    if (titleStr.isEmpty()) titleStr = groupsVarMap.value("title").toString();
    group->setTitle(titleStr);
    group->setConfigData(groupsVarMap);
    group->setState(this->state());

    connect(group, SIGNAL(configChanged()), this, SLOT(onDataChanged()));
    connect(group, SIGNAL(urlRequested(QString, QVariant)), this, SIGNAL(urlRequested(QString, QVariant)));
    connect(group, SIGNAL(groupDeleted(TopWin8DesktopGroup*)), this, SLOT(deleteGroup(TopWin8DesktopGroup*)));
    connect(group, SIGNAL(layoutedRequest()), this, SLOT(doLayout()));

    return group;
}

void TopWin8Desktop::mouseReleaseEvent(QMouseEvent *iEvent)
{
    Q_D(TopWin8Desktop);
    if (iEvent->button() == Qt::RightButton)
    {
        if (d->desktopMenu != nullptr)
        {
            if (!isDataModified() && state() == TopDesktopItemAbs::State_Editing)
            {
                d->desktopActions.value(DesktopAction_SetEditable)->setText(tr("Cancel Edit"));
                d->desktopActions.value(DesktopAction_SetEditable)->setIcon(TRES->icon(""));
            }
            else
            {
                d->desktopActions.value(DesktopAction_SetEditable)->setText(tr("Edit Desktop"));
                d->desktopActions.value(DesktopAction_SetEditable)->setIcon(TRES->icon(""));
            }

            d->desktopActions.value(DesktopAction_SetEditable)->setDisabled((state() == TopDesktopItemAbs::State_Editing) && isDataModified());
            d->desktopActions.value(DesktopAction_SaveModified)->setDisabled(!isDataModified());
            d->desktopActions.value(DesktopAction_SetBackground)->setDisabled(!(state() == TopDesktopItemAbs::State_Editing));
            d->desktopActions.value(DesktopAction_AddGroup)->setDisabled(!(state() == TopDesktopItemAbs::State_Editing));
            d->desktopMenu->setProperty("POSITION", iEvent->pos());
            d-> desktopMenu->exec(QCursor::pos());
        }
    }
    return QFrame::mouseReleaseEvent(iEvent);
}

void TopWin8Desktop::wheelEvent(QWheelEvent *iEvent)
{
    Q_D(TopWin8Desktop);
    int deltaInt = iEvent->delta();
    QScrollBar *scrl = d->scrollArea->horizontalScrollBar();
    scrl->setValue(scrl->value() - deltaInt);

    QFrame::wheelEvent(iEvent);
}

void TopWin8Desktop::dragEnterEvent(QDragEnterEvent *iEvent)
{
    if (iEvent->mimeData()->hasFormat("application/desktop-groupdata"))
    {
        iEvent->setDropAction(Qt::MoveAction);
        iEvent->accept();
    }
    else
    {
        iEvent->ignore();
    }
}

void TopWin8Desktop::dropEvent(QDropEvent *iEvent)
{
    Q_D(TopWin8Desktop);
    if (iEvent->mimeData()->hasFormat("application/desktop-groupdata"))
    {
        QByteArray categoryDataBytLst = iEvent->mimeData()->data("application/desktop-groupdata");
        QDataStream dataStream(&categoryDataBytLst, QIODevice::ReadOnly);

        QPoint offsetPos;
        QString titleStr;
        QSize size, cellSize;

        QString categoryStr;
        QVariantMap categoryDataVarMap;
        dataStream >> offsetPos >> titleStr >> size >> cellSize >> categoryStr;
        categoryDataVarMap = TDataParse::jsonStr2Variant(categoryStr).toMap();

        TopWin8DesktopGroup *newCate = createGroup(categoryDataVarMap);
        // 关闭自动删除
        newCate->setAttribute(Qt::WA_DeleteOnClose);
        newCate->setDockArea("position");

        QPoint atWgtPoint = classPointToCenterWidgetPoint(iEvent->pos() - offsetPos);
        if (atWgtPoint.x() < 0) atWgtPoint.setX(0);
        if (atWgtPoint.y() < 0) atWgtPoint.setY(0);
        newCate->setPosition(atWgtPoint);

        // 在最后触发
        d->groupsLst.removeOne(qobject_cast<TopWin8DesktopGroup *>(iEvent->source()));
        iEvent->setDropAction(Qt::MoveAction);
        iEvent->accept();

        d->groupsLst.append(newCate);
        doLayout();
        onDataChanged();
    }
}

void TopWin8Desktop::resizeEvent(QResizeEvent *)
{
    doLayout();
}

QPoint TopWin8Desktop::classPointToCenterWidgetPoint(QPoint iPoint)
{
    Q_D(TopWin8Desktop);
    return iPoint - d->scrollArea->pos() - d->scrollCenterWgt->pos();
}

TopWin8Desktop::DockArea TopWin8Desktop::str2DockArea(const QString &iSequenceStr)
{
    if (iSequenceStr == "topLeft")          return DockArea::DockArea_TopLeft;
    else if (iSequenceStr == "top")         return DockArea::DockArea_Top;
    else if (iSequenceStr == "topRight")    return DockArea::DockArea_TopRight;
    else if (iSequenceStr == "left")        return DockArea::DockArea_Left;
    else if (iSequenceStr == "center")      return DockArea::DockArea_Center;
    else if (iSequenceStr == "right")       return DockArea::DockArea_Right;
    else if (iSequenceStr == "bottomLeft")  return DockArea::DockArea_BottomLeft;
    else if (iSequenceStr == "bottom")      return DockArea::DockArea_Bottom;
    else if (iSequenceStr == "bottomRight") return DockArea::DockArea_BottomRight;

    return DockArea::DockArea_Position;
}

TopWin8Desktop::Alignment TopWin8Desktop::str2Alignment(const QString &iAlignmentStr)
{
    if (iAlignmentStr == "vertical")          return Alignment::Alignment_Vertical;
    else if (iAlignmentStr == "horizontal")   return Alignment::Alignment_Horizontal;
    return Alignment::Alignment_None;
}

TopWin8Desktop::Alignment TopWin8Desktop::alignmentAtDockArea(TopWin8Desktop::DockArea iDockArea)
{
    if (iDockArea == DockArea_TopLeft)  return str2Alignment(config("topLeft").toString());
    else if (iDockArea == DockArea_Top) return str2Alignment(config("top").toString());
    else if (iDockArea == DockArea_TopRight) return str2Alignment(config("topRight").toString());
    else if (iDockArea == DockArea_Left) return str2Alignment(config("left").toString());
    else if (iDockArea == DockArea_Center) return str2Alignment(config("center").toString());
    else if (iDockArea == DockArea_Right) return str2Alignment(config("right").toString());
    else if (iDockArea == DockArea_BottomLeft) return str2Alignment(config("bottomLeft").toString());
    else if (iDockArea == DockArea_Bottom) return str2Alignment(config("bottom").toString());
    else if (iDockArea == DockArea_BottomRight) return str2Alignment(config("bottomRight").toString());

    return Alignment::Alignment_None;
}

TopDesktopItemAbs::ClickType TopWin8Desktop::str2ModuleClickType(const QString &iClickTypeStr) const
{
    QString clickTypeStr = iClickTypeStr.toLower().trimmed();
    if (clickTypeStr == "click")
    {
        return TopDesktopItemAbs::ClickType_Click;
    }
    else
    {
        return TopDesktopItemAbs::ClickType_DoubleClick;
    }
}

QString TopWin8Desktop::moduleClickType2Str(TopDesktopItemAbs::ClickType iClickType) const
{
    if (iClickType == TopDesktopItemAbs::ClickType_Click)
    {
        return "click";
    }
    else
    {
        return "dclick";
    }
}

int TopWin8Desktop::bottomYAtDockArea(TopWin8Desktop::DockArea iDockArea,
                                      QList<TopWin8DesktopGroup *> &iGroupsLst,
                                      int iStartYInt)
{
    int bottomYInt = 0;
    int maxHeightInt = 0;
    Alignment alignment = alignmentAtDockArea(iDockArea);

    if (alignment == Alignment_None) alignment = Alignment_Horizontal;

    if (alignment == Alignment_Horizontal)
    {
        foreach (TopWin8DesktopGroup *category, iGroupsLst) {
            if (str2DockArea(category->dockArea()) != iDockArea) continue;
            if (category->height() > maxHeightInt) maxHeightInt = category->height();
        }
        bottomYInt = iStartYInt + maxHeightInt + WndSize_GroupsVSpace;
    }
    else if (alignment == Alignment_Vertical)
    {
        foreach (TopWin8DesktopGroup *category, iGroupsLst) {
            if (str2DockArea(category->dockArea()) != iDockArea) continue;
            maxHeightInt += category->height() + WndSize_GroupsVSpace;
        }
        bottomYInt = iStartYInt + maxHeightInt;
    }

    return bottomYInt;
}

int TopWin8Desktop::rightXAtDockArea(TopWin8Desktop::DockArea iDockArea,
                                     QList<TopWin8DesktopGroup *> &iGroupsLst,
                                     int iStratXInt)
{
    int rightXInt = 0;
    int maxWidthInt = 0;
    Alignment alignment = alignmentAtDockArea(iDockArea);

    if (alignment == Alignment_None) alignment = Alignment_Horizontal;

    if (alignment == Alignment_Horizontal)
    {
        foreach (TopWin8DesktopGroup *category, iGroupsLst) {
            if (str2DockArea(category->dockArea()) != iDockArea) continue;
            maxWidthInt += category->width() + WndSize_GroupsHSpace;
        }
        rightXInt = iStratXInt + maxWidthInt;
    }
    else if (alignment == Alignment_Vertical)
    {
        foreach (TopWin8DesktopGroup *category, iGroupsLst) {
            if (str2DockArea(category->dockArea()) != iDockArea) continue;
            if (category->width() > maxWidthInt) maxWidthInt = category->width();;
        }
        rightXInt = iStratXInt + maxWidthInt + WndSize_GroupsHSpace;
    }

    return rightXInt;
}

void TopWin8Desktop::layoutDockAreaGroup(QSize /*iMinWgtSize*/)
{
    Q_D(TopWin8Desktop);
    QRect rect(0,0,0,0);
    int tmpInt = 0;
    // 1.计算topLeft,top,topRight层最底位置
    int topBottomY = bottomYAtDockArea(DockArea_TopLeft, d->groupsLst, rect.y());
    tmpInt = bottomYAtDockArea(DockArea_Top, d->groupsLst, rect.y());
    if (tmpInt > topBottomY) topBottomY = tmpInt;
    tmpInt = bottomYAtDockArea(DockArea_TopRight, d->groupsLst, rect.y());
    if (tmpInt > topBottomY) topBottomY = tmpInt;

    // 2.计算left,center,right的最最底位置
    int centerBottomY = 0;
    centerBottomY = bottomYAtDockArea(DockArea_Left, d->groupsLst, topBottomY);
    tmpInt = bottomYAtDockArea(DockArea_Center, d->groupsLst, topBottomY);
    if (tmpInt > centerBottomY) centerBottomY = tmpInt;
    tmpInt = bottomYAtDockArea(DockArea_Right, d->groupsLst, topBottomY);
    if (tmpInt > centerBottomY) centerBottomY = tmpInt;

    // 3.计算bottomLeft,bottom,bottomRight的最低位置
    int bottomBottomY = 0;
    bottomBottomY = bottomYAtDockArea(DockArea_BottomLeft, d->groupsLst, centerBottomY);
    tmpInt = bottomYAtDockArea(DockArea_Bottom, d->groupsLst, centerBottomY);
    if (tmpInt > bottomBottomY) bottomBottomY = tmpInt;
    tmpInt = bottomYAtDockArea(DockArea_BottomRight, d->groupsLst, centerBottomY);
    if (tmpInt > bottomBottomY) bottomBottomY = tmpInt;

    // 4.计算topLeft,left,bottomLeft的最右边
    int leftRightInt = 0;
    leftRightInt = rightXAtDockArea(DockArea_TopLeft, d->groupsLst, rect.x());
    tmpInt = rightXAtDockArea(DockArea_Left, d->groupsLst, rect.x());
    if (tmpInt > leftRightInt) leftRightInt = tmpInt;
    tmpInt = rightXAtDockArea(DockArea_BottomLeft, d->groupsLst, rect.x());
    if (tmpInt > leftRightInt) leftRightInt = tmpInt;

    // 5.计算top,center,bottom的右边位置
    int centerRightInt = 0;
    centerRightInt = rightXAtDockArea(DockArea_Top, d->groupsLst, leftRightInt);
    tmpInt = rightXAtDockArea(DockArea_Center, d->groupsLst, leftRightInt);
    if (tmpInt > centerRightInt) centerRightInt = tmpInt;
    tmpInt = rightXAtDockArea(DockArea_Bottom, d->groupsLst, leftRightInt);
    if (tmpInt > centerRightInt) centerRightInt = tmpInt;

    // 6.计算TopRight,Right,bottomRight 的最右边位置
    int rightRightInt = 0;
    rightRightInt = rightXAtDockArea(DockArea_TopRight, d->groupsLst, centerRightInt);
    tmpInt = rightXAtDockArea(DockArea_Right, d->groupsLst, centerRightInt);
    if (tmpInt > rightRightInt) rightRightInt = tmpInt;
    tmpInt = rightXAtDockArea(DockArea_BottomRight, d->groupsLst, centerRightInt);
    if (tmpInt > rightRightInt) rightRightInt = tmpInt;

    // ①.排列topLeft,left,bottomLeft
    putGroupsAtDockArea(DockArea_TopLeft, d->groupsLst, rect.topLeft());
    if (alignmentAtDockArea(DockArea_TopLeft) == Alignment::Alignment_Vertical)
    {
        putGroupsAtDockArea(DockArea_Left, d->groupsLst, QPoint(rect.x(), topBottomY));
    }
    else
    {
        putGroupsAtDockArea(DockArea_Left, d->groupsLst, QPoint(rect.x(), (topBottomY + centerBottomY)/2));
    }

    putGroupsAtDockArea(DockArea_BottomLeft, d->groupsLst, QPoint(rect.x(), bottomBottomY));


    // ②.排列Top,Center,Bottom
    if (alignmentAtDockArea(DockArea_Top) == Alignment_Vertical)
    {
        putGroupsAtDockArea(DockArea_Top, d->groupsLst, QPoint((leftRightInt + centerRightInt)/2, rect.y()));
    }
    else
    {
        putGroupsAtDockArea(DockArea_Top, d->groupsLst, QPoint(leftRightInt, rect.y()));
    }

    if (alignmentAtDockArea(DockArea_Center) == Alignment_Vertical)
    {
        putGroupsAtDockArea(DockArea_Center, d->groupsLst, QPoint((leftRightInt + centerRightInt)/2, topBottomY));
    }
    else
    {
        putGroupsAtDockArea(DockArea_Center, d->groupsLst, QPoint(leftRightInt, (centerBottomY + topBottomY) /2));
    }

    if (alignmentAtDockArea(DockArea_Bottom) == Alignment_Vertical)
    {
        putGroupsAtDockArea(DockArea_Bottom, d->groupsLst, QPoint((leftRightInt + centerRightInt)/2, bottomBottomY));
    }
    else
    {
        putGroupsAtDockArea(DockArea_Bottom, d->groupsLst, QPoint(leftRightInt, bottomBottomY));
    }

    // ③ 排列TopRight,Right,bottomRight
    putGroupsAtDockArea(DockArea::DockArea_TopRight, d->groupsLst, QPoint(rightRightInt, rect.y()));
    if (alignmentAtDockArea(DockArea::DockArea_Right) == Alignment_Vertical)
    {
        putGroupsAtDockArea(DockArea::DockArea_Right, d->groupsLst, QPoint(rightRightInt, topBottomY));
    }
    else
    {
        putGroupsAtDockArea(DockArea::DockArea_Right, d->groupsLst, QPoint(rightRightInt, (topBottomY + centerBottomY) /2));
    }
    putGroupsAtDockArea(DockArea::DockArea_BottomRight, d->groupsLst, QPoint(rightRightInt, bottomBottomY));
    // 重算centerWgt最大位置,
}

void TopWin8Desktop::putGroupsAtDockArea(TopWin8Desktop::DockArea iDockArea,
                                         QList<TopWin8DesktopGroup *> &iLayoutedGroupsLst,
                                         QPoint iAlignmentPosition)
{
    Alignment alignment = alignmentAtDockArea(iDockArea);
    if (alignment == Alignment::Alignment_None) alignment = Alignment::Alignment_Horizontal;

    if (iDockArea == DockArea_Position) return;

    QList<TopWin8DesktopGroup *> orderedGroupsLst = groupsOrderByOneDockArea(iDockArea, iLayoutedGroupsLst);
    QPoint curAlignmentPosition = iAlignmentPosition;
    QPoint topLeftPosition = iAlignmentPosition;

    if (alignment == Alignment::Alignment_Horizontal) // 水平左右排
    {
        switch (iDockArea)
        {
        case DockArea::DockArea_TopLeft:
        case DockArea::DockArea_Top:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                // ←
                topLeftPosition = curAlignmentPosition;
                category->move(topLeftPosition);
                curAlignmentPosition.setX(category->geometry().right() + WndSize_GroupsHSpace);
            }
            break;
        case DockArea::DockArea_TopRight:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                // ←]
                topLeftPosition = curAlignmentPosition;
                topLeftPosition.setX(curAlignmentPosition.x() - category->width());
                category->move(topLeftPosition);
                curAlignmentPosition.setX(category->geometry().left() - WndSize_GroupsHSpace);
            }
            break;

        case DockArea::DockArea_Left:
        case DockArea::DockArea_Center:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                topLeftPosition = curAlignmentPosition;
                topLeftPosition.setY(curAlignmentPosition.y() - category->height() / 2);
                category->move(topLeftPosition);
                curAlignmentPosition.setX(category->geometry().right() + WndSize_GroupsHSpace);
            }
            break;
        case DockArea::DockArea_Right:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                topLeftPosition.setY(curAlignmentPosition.y() - category->height() / 2);
                topLeftPosition.setX(curAlignmentPosition.x() - category->width());
                category->move(topLeftPosition);
                curAlignmentPosition.setX(category->geometry().left() - WndSize_GroupsHSpace);
            }
            break;

        case DockArea::DockArea_BottomLeft:
        case DockArea::DockArea_Bottom:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                topLeftPosition = curAlignmentPosition;
                topLeftPosition.setY(curAlignmentPosition.y() - category->height());
                category->move(topLeftPosition);
                curAlignmentPosition.setX(category->geometry().right() + WndSize_GroupsHSpace);
            }
            break;
        case DockArea::DockArea_BottomRight:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                topLeftPosition.setY(curAlignmentPosition.y() - category->height());
                topLeftPosition.setX(curAlignmentPosition.x() - category->width());
                category->move(topLeftPosition);
                curAlignmentPosition.setX(category->geometry().left() - WndSize_GroupsHSpace);
            }
            break;
        case DockArea::DockArea_Position:
            break;
        }
    }
    else if (alignment == Alignment_Vertical)   // here
    {
        switch (iDockArea)
        {
        case DockArea::DockArea_TopLeft:
        case DockArea::DockArea_Left:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                // ←
                topLeftPosition = curAlignmentPosition;
                category->move(topLeftPosition);
                curAlignmentPosition.setY(category->geometry().bottom() + WndSize_GroupsVSpace);
            }
            break;
        case DockArea::DockArea_Top:
        case DockArea::DockArea_Center:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                topLeftPosition = curAlignmentPosition;
                topLeftPosition.setX(curAlignmentPosition.x() - category->width() / 2);
                category->move(topLeftPosition);
                curAlignmentPosition.setY(category->geometry().bottom() + WndSize_GroupsVSpace);
            }
            break;
        case DockArea::DockArea_TopRight:
        case DockArea::DockArea_Right:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                // ←
                topLeftPosition = curAlignmentPosition;
                topLeftPosition.setX(curAlignmentPosition.x() - category->width());
                category->move(curAlignmentPosition);
                curAlignmentPosition.setY(category->geometry().bottom() + WndSize_GroupsVSpace);
            }
            break;
        case DockArea::DockArea_BottomLeft:

            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                topLeftPosition = curAlignmentPosition;
                topLeftPosition.setY(curAlignmentPosition.y() - category->height());
                category->move(topLeftPosition);
                curAlignmentPosition.setY(category->geometry().top() - WndSize_GroupsVSpace);
            }
            break;

        case DockArea::DockArea_Bottom:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                topLeftPosition.setX(curAlignmentPosition.x() - category->width() / 2);
                topLeftPosition.setY(curAlignmentPosition.y() - category->height());
                category->move(topLeftPosition);
                curAlignmentPosition.setY(category->geometry().top() - WndSize_GroupsVSpace);
            }
            break;
        case DockArea::DockArea_BottomRight:
            foreach (TopWin8DesktopGroup *category, orderedGroupsLst) {
                topLeftPosition.setY(curAlignmentPosition.y() - category->height());
                topLeftPosition.setX(curAlignmentPosition.x() - category->width());
                category->move(topLeftPosition);
                curAlignmentPosition.setY(category->geometry().top() - WndSize_GroupsVSpace);
            }
            break;
        case DockArea::DockArea_Position:
            break;
        }
    }

}

QList<TopWin8DesktopGroup *> TopWin8Desktop::groupsOrderByOneDockArea(TopWin8Desktop::DockArea iDockArea, QList<TopWin8DesktopGroup *> &iLayoutedGroupsLst)
{
    QList<TopWin8DesktopGroup *> groupsAtOneDockAreaLst;
    QList<TopWin8DesktopGroup *> orderedGroupsAtOneDockAreaLst;
    foreach (TopWin8DesktopGroup *group, iLayoutedGroupsLst) {
        if (str2DockArea(group->dockArea()) != iDockArea) continue;
        groupsAtOneDockAreaLst.append(group);
    }

    int orderedNumsInt;
    int curIndexInt = 0;
    foreach (TopWin8DesktopGroup *group, groupsAtOneDockAreaLst) {
        orderedNumsInt = orderedGroupsAtOneDockAreaLst.count();
        for (curIndexInt = 0; curIndexInt < orderedNumsInt; ++curIndexInt)
        {
            if (group->dockSequence() < orderedGroupsAtOneDockAreaLst.at(curIndexInt)->dockSequence())
            {
                orderedGroupsAtOneDockAreaLst.insert(curIndexInt, group);
                break;
            }
        }

        if (curIndexInt >= orderedNumsInt - 1)
        {
            orderedGroupsAtOneDockAreaLst.append(group);
        }
    }
    return orderedGroupsAtOneDockAreaLst;
}

QSize TopWin8Desktop::getDockAreaSize(TopWin8Desktop::Alignment iAlignment, const QList<TopWin8DesktopGroup *> &iGroupsLst)
{
    int widthInt = 0;
    int heightInt = 0;

    if (iAlignment == Alignment_Horizontal)
    {
        foreach (TopWin8DesktopGroup *category, iGroupsLst) {
            if (category->height() > heightInt) heightInt = category->height();
            widthInt += category->width();
        }
        if (iGroupsLst.count() > 0)
        {
            widthInt += (iGroupsLst.count() - 1) * + WndSize_GroupsHSpace;
        }

    }
    else
    {
        foreach (TopWin8DesktopGroup *category, iGroupsLst) {
            if (category->width() > widthInt) widthInt = category->width();
            heightInt += category->height();
        }
        if (iGroupsLst.count() > 0)
        {
            heightInt += (iGroupsLst.count() - 1) * + WndSize_GroupsHSpace;
        }
    }

    return QSize(widthInt, heightInt);
}

QSize TopWin8Desktop::calcDesktopSize()
{
    QSize centerSize = getDockAreaSize(alignmentAtDockArea(DockArea_Center), getDockAreaGroupList(DockArea_Center));
    QRect centerRect = QRect(centerSize.width()/2, centerSize.height()/2, centerSize.width(), centerSize.height());

    QSize topSize = getDockAreaSize(alignmentAtDockArea(DockArea_Top), getDockAreaGroupList(DockArea_Top));
    QRect topRect = QRect(-topSize.width()/2, centerRect.top() - topSize.height() - WndSize_GroupsHSpace, topSize.width(), topSize.height());

    QSize botSize = getDockAreaSize(alignmentAtDockArea(DockArea_Bottom), getDockAreaGroupList(DockArea_Bottom));
    QRect botRect = QRect(-topSize.width()/2, centerRect.bottom() - WndSize_GroupsHSpace, botSize.width(), botSize.height());

    QRect unitedRect = centerRect.united(topRect).united(botRect);

    QSize leftSize = getDockAreaSize(alignmentAtDockArea(DockArea_Left), getDockAreaGroupList(DockArea_Left));
    QRect leftRect = QRect(centerRect.left() - leftSize.width() - WndSize_GroupsVSpace, -leftSize.height()/2, leftSize.width(), leftSize.height());

    QList<QRect> rectList;
    rectList << centerRect << topRect << botRect << leftRect;
    QSize topLeftSize = getDockAreaSize(alignmentAtDockArea(DockArea_TopLeft), getDockAreaGroupList(DockArea_TopLeft));
    QRect topLeftRect = QRect(unitedRect.left() - topLeftSize.width() - WndSize_GroupsVSpace, unitedRect.top(), topLeftSize.width(), topLeftSize.height());
    while(true)
    {
        bool insertectBol = false;
        foreach (QRect otherRect, rectList)
        {
            if (topLeftRect.intersects(otherRect)){
                insertectBol = true;
                break;
            }
        }
        if (!insertectBol) break;
        topLeftRect.translate(0, -50);
    }

    QSize botLeftSize = getDockAreaSize(alignmentAtDockArea(DockArea_BottomLeft), getDockAreaGroupList(DockArea_BottomLeft));
    QRect botLeftRect = QRect(unitedRect.left() - botLeftSize.width() - WndSize_GroupsVSpace, unitedRect.bottom() + botLeftSize.height(), botLeftSize.width(), botLeftSize.height());
    while(true)
    {
        bool insertectBol = false;
        foreach (QRect otherRect, rectList)
        {
            if (botLeftRect.intersects(otherRect)){
                insertectBol = true;
                break;
            }
        }
        if (!insertectBol) break;
        botLeftRect.translate(0, 50);
    }


    QSize rightSize = getDockAreaSize(alignmentAtDockArea(DockArea_Right), getDockAreaGroupList(DockArea_Right));
    QRect rightRect = QRect(centerRect.right() + WndSize_GroupsVSpace, -rightSize.height()/2, rightSize.width(), rightSize.height());

    rectList.clear();
    rectList << centerRect << topRect << botRect << rightRect;
    QSize topRightSize = getDockAreaSize(alignmentAtDockArea(DockArea_TopRight), getDockAreaGroupList(DockArea_TopRight));
    QRect topRightRect = QRect(unitedRect.right() + WndSize_GroupsVSpace, unitedRect.top(), topRightSize.width(), topRightSize.height());
    while(true)
    {
        bool insertectBol = false;
        foreach (QRect otherRect, rectList)
        {
            if (topRightRect.intersects(otherRect)){
                insertectBol = true;
                break;
            }
        }
        if (!insertectBol) break;
        topRightRect.translate(0, -50);
    }

    QSize botRightSize = getDockAreaSize(alignmentAtDockArea(DockArea_BottomRight), getDockAreaGroupList(DockArea_BottomRight));
    QRect botRightRect = QRect(unitedRect.right() + WndSize_GroupsVSpace, unitedRect.bottom() + botRightSize.height(), botRightSize.width(), botRightSize.height());
    while(true)
    {
        bool insertectBol = false;
        foreach (QRect otherRect, rectList)
        {
            if (botRightRect.intersects(otherRect)){
                insertectBol = true;
                break;
            }
        }
        if (!insertectBol) break;
        botRightRect.translate(0, 50);
    }



    return centerRect.united(topRect).united(botRect).united(leftRect).united(rightRect)
            .united(topLeftRect).united(topRightRect).united(botLeftRect).united(botRightRect)
            .size();

}

QList<TopWin8DesktopGroup *> TopWin8Desktop::getDockAreaGroupList(TopWin8Desktop::DockArea iDockArea)
{
    Q_D(TopWin8Desktop);
    QMap<double, TopWin8DesktopGroup *> grpMap;
    int i = 0;
    foreach (TopWin8DesktopGroup *grp, d->groupsLst)
    {
        if (str2DockArea(grp->dockArea()) == iDockArea)
        {
            i++;
            grpMap.insert(QString("%1.%2").arg(grp->dockSequence()).arg(i).toDouble(), grp);
        }
    }
    return grpMap.values();
}

void TopWin8Desktop::layoutDockAreaGroup()
{
    Q_D(TopWin8Desktop);
    QPoint topLeftPos(0, 0);
    foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_TopLeft)) {
        grp->move(topLeftPos);
        if (alignmentAtDockArea(DockArea_TopLeft) == Alignment_Horizontal) topLeftPos.setX(topLeftPos.x() + grp->width() + WndSize_GroupsHSpace);
        else topLeftPos.setY(topLeftPos.y() + grp->height() + WndSize_GroupsVSpace);
    }

    if (Alignment_Horizontal == alignmentAtDockArea(DockArea_TopRight))
    {
        int x = d->scrollCenterWgt->width();
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_TopRight)) {
            grp->move(QPoint(x - grp->width(), 0));
            x -= grp->width() + WndSize_GroupsHSpace;
        }
    }
    else
    {
        int y = 0;
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_TopRight)) {
            grp->move(d->scrollCenterWgt->width() - grp->width(), y);
            y += grp->height() + WndSize_GroupsVSpace;
        }
    }

    if (Alignment_Horizontal == alignmentAtDockArea(DockArea_BottomLeft))
    {
        int x = 0;
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_BottomLeft)) {
            grp->move(x, d->scrollCenterWgt->height() - grp->height());
            x += grp->width() + WndSize_GroupsHSpace;
        }
    }
    else
    {
        int y = d->scrollCenterWgt->height();
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_BottomLeft)) {
            grp->move(0, y - grp->height());
            y -= grp->height() + WndSize_GroupsVSpace;
        }
    }

    if (Alignment_Horizontal == alignmentAtDockArea(DockArea_BottomRight))
    {
        int x = d->scrollCenterWgt->width();
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_BottomRight)) {
            grp->move(QPoint(x - grp->width(), d->scrollCenterWgt->height() - grp->height()));
            x -= grp->width() + WndSize_GroupsHSpace;
        }
    }
    else
    {
        int y = d->scrollCenterWgt->height();
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_BottomRight)) {
            grp->move(d->scrollCenterWgt->width() - grp->width(), y - grp->height());
            y -= grp->height() + WndSize_GroupsVSpace;
        }
    }



    if (Alignment_Horizontal == alignmentAtDockArea(DockArea_Top))
    {
        QSize topSize = getDockAreaSize(Alignment_Horizontal, getDockAreaGroupList(DockArea_Top));
        QPoint topPos(d->scrollCenterWgt->width() / 2 - topSize.width() / 2 , 0);
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_Top)) {
            grp->move(topPos);
            topPos += QPoint(grp->width() + WndSize_GroupsHSpace, 0);
        }
    }
    else
    {
        int y = 0;
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_Top)) {
            grp->move(QPoint(d->scrollCenterWgt->width() / 2 - grp->size().width() / 2, y));
            y += grp->height() + WndSize_GroupsVSpace;
        }
    }

    if (Alignment_Horizontal == alignmentAtDockArea(DockArea_Left))
    {
        int x = 0;
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_Left)) {
            grp->move(x, d->scrollCenterWgt->height() / 2 - grp->height() / 2);
            x += grp->width() + WndSize_GroupsHSpace;
        }
    }
    else
    {
        QSize leftSize = getDockAreaSize(Alignment_Vertical, getDockAreaGroupList(DockArea_Left));
        int y = d->scrollCenterWgt->height() / 2 - leftSize.height() / 2;
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_Left)) {
            grp->move(0, y);
            y += grp->height() + WndSize_GroupsVSpace;
        }
    }

    if (Alignment_Horizontal == alignmentAtDockArea(DockArea_Right))
    {
        int x = d->scrollCenterWgt->width();
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_Right)) {
            grp->move(x - grp->width(), d->scrollCenterWgt->height() / 2 - grp->height() / 2);
            x -= grp->width() + WndSize_GroupsHSpace;
        }
    }
    else
    {
        QSize rightSize = getDockAreaSize(Alignment_Vertical, getDockAreaGroupList(DockArea_Right));
        int y = d->scrollCenterWgt->height() / 2 - rightSize.height() / 2;
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_Right)) {
            grp->move(d->scrollCenterWgt->width() - grp->width(), y);
            y += grp->height() + WndSize_GroupsVSpace;
        }
    }


    if (Alignment_Horizontal == alignmentAtDockArea(DockArea_Bottom))
    {
        QSize botSize = getDockAreaSize(Alignment_Horizontal, getDockAreaGroupList(DockArea_Bottom));
        int x = d->scrollCenterWgt->width() / 2 - botSize.width() / 2;
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_Bottom)) {
            grp->move(x, d->scrollCenterWgt->height() - grp->height());
            x += grp->width() + WndSize_GroupsHSpace;
        }
    }
    else
    {
        int y = d->scrollCenterWgt->height();
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_Bottom)) {
            grp->move(d->scrollCenterWgt->width() / 2 - grp->size().width() / 2, y - grp->height());
            y -= grp->height() + WndSize_GroupsVSpace;
        }
    }

    if (Alignment_Horizontal == alignmentAtDockArea(DockArea_Center))
    {
        QSize centerSize = getDockAreaSize(Alignment_Horizontal, getDockAreaGroupList(DockArea_Center));
        int x = d->scrollCenterWgt->width() / 2 - centerSize.width() / 2;
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_Center)) {
            grp->move(x, d->scrollCenterWgt->height() / 2 - grp->height() / 2);
            x += grp->width() + WndSize_GroupsHSpace;
        }
    }
    else
    {
        QSize centerSize = getDockAreaSize(Alignment_Vertical, getDockAreaGroupList(DockArea_Center));
        int y = d->scrollCenterWgt->height() / 2 - centerSize.height() / 2;
        foreach (TopWin8DesktopGroup *grp, getDockAreaGroupList(DockArea_Center)) {
            grp->move(d->scrollCenterWgt->width() / 2 - grp->size().width() / 2, y);
            y += grp->height() + WndSize_GroupsVSpace;
        }
    }

}

