﻿#include "moduleconfpanel.h"
#include <QDesktopServices>
#include <QFileSystemWatcher>
#include <QTimer>
#include <QToolBar>
#include <QVBoxLayout>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tlogger.h>
#include <tbaseutil/tfileio.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topcore.h>
#include <toputil/t.h>
#include <twidget/ttextedit.h>
#include <twidget/tmessagebox.h>
#include "moduleconfmgtthread.h"

#define TABLE_SYS_SETTING_MODULE        "sys_setting_module"
#define TABLE_SYS_SETTING_ACTION        "sys_setting_action"
#define TABLE_SYS_SETTING_PERMISSION    "sys_setting_permission"
#define TABLE_SYS_SETTING_ENUM          "sys_setting_enum"

SysModuleConfPanel::SysModuleConfPanel(const QString &iModuleNameStr, const QVariantMap &iUrlPars, QWidget *iParent)
    :TopClassAbs(iParent)
{
    this->appendLanguage("module-conf-panel");
    this->setLicenseKey("");
    this->initModule(iModuleNameStr, iUrlPars);
    this->setStyleSheet(".ModuleConfPanel{background-color:#F5FAFD;}");

    initTextEdit();
    refreshActionState();
    QTimer::singleShot(0, this, &SysModuleConfPanel::refresh);
}

SysModuleConfPanel::~SysModuleConfPanel()
{
}

void SysModuleConfPanel::refresh()
{
}

void SysModuleConfPanel::setToolbarVisible(bool iVisibleBol)
{
    if (mToolBar != nullptr) {
        mToolBar->setVisible(iVisibleBol);
    }
}

void SysModuleConfPanel::setTextReadOnly(bool iReadOnlyBol)
{
    mTextEdit->setReadOnly(iReadOnlyBol);
    mActTextEdit->setReadOnly(iReadOnlyBol);
}

void SysModuleConfPanel::openWithOtherProgram()
{
    const QString tempFilePath = QStandardPaths::writableLocation(QStandardPaths::TempLocation)
            + "/b4245fc6-556b-4343-a19b-74f0d0b41800_MODULE_TEMP_FILE";
    if (!TFileIo::writeFileString(mTextEdit->toPlainText(), tempFilePath)) {
        alertError(ttr("Write content of TextEdit to temp file failed."));
        return;
    }
    const QString programPath = config("outside_process_path").toString();
    if (QFile::exists(programPath)) {
        if (!QProcess::startDetached(programPath, QStringList() << tempFilePath)) {
            alertError(ttr("Execute program failed."));
            return;
        }
    } else {
        if (!QDesktopServices::openUrl(tempFilePath)) {
            alertError(ttr("Open url failed."));
            return;
        }
    }
    if (mFileWatcher == nullptr) {
        mFileWatcher = new QFileSystemWatcher(this);
        connect(mFileWatcher, &QFileSystemWatcher::fileChanged, [this](const QString &path){
            if (mTextEdit != nullptr) {
                mTextEdit->setPlainText(TFileIo::readFileString(path));
            }
        });
    }
    mFileWatcher->addPath(tempFilePath);
}

void SysModuleConfPanel::loadData(const QVariantMap &iDataMap)
{
    bool isAction = false;
    if ("action" == iDataMap.value("type").toString()) {
        mActTextEdit->setVisible(true);
        isAction = true;
    } else {
        mActTextEdit->setVisible(false);
    }

    if (isAction) {
        mTextEdit->setText(iDataMap.value("prog").toString());
        mActTextEdit->setText(iDataMap.value("attr").toString());
    } else {
        mTextEdit->setText(iDataMap.value("text").toString());
    }
    this->setMyModuleName(iDataMap.value("module_name").toString());
    this->setMyName(iDataMap.value("name").toString());
    this->setMyType(iDataMap.value("type").toString());
    setDataModified(false);
}

void SysModuleConfPanel::saveAdmin()
{
    save("admin");
}

void SysModuleConfPanel::saveDev()
{
    save("dev");
}

void SysModuleConfPanel::saveProd()
{
    save("prod");
}

void SysModuleConfPanel::saveProject()
{
    save("project");
}

void SysModuleConfPanel::saveUser()
{
    save("user");
}

void SysModuleConfPanel::cancelAdmin()
{
    cancel("admin");
}

void SysModuleConfPanel::cancelDev()
{
    cancel("dev");
}

void SysModuleConfPanel::cancelProd()
{
    cancel("prod");
}

void SysModuleConfPanel::cancelProject()
{
    cancel("project");
}

void SysModuleConfPanel::cancelUser()
{
    cancel("user");
}

void SysModuleConfPanel::onDataChange()
{
    setDataModified();
    emit dataChanged(QVariant());
}

void SysModuleConfPanel::initTextEdit()
{
    mTextEdit = new TTextEdit(this);
    mActTextEdit = new TTextEdit(this);
    mActTextEdit->setVisible(false);
    connect(mTextEdit, SIGNAL(textChanged()), this, SLOT(onDataChange()));
    connect(mActTextEdit, SIGNAL(textChanged()), this, SLOT(onDataChange()));
}

void SysModuleConfPanel::initWidget()
{
    QWidget *centerWgt = new QWidget(this);
    QVBoxLayout *vboxlayout = new QVBoxLayout(centerWgt);
    vboxlayout->setMargin(0);
    vboxlayout->setSpacing(0);
    this->setCentralWidget(centerWgt);
	
	QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget(QString("MAIN_%1_TOOLBAR").arg(mRank.toUpper())));
    if (toolbar != nullptr) {
        toolbar->setWindowTitle(ttr("ToolBar"));
        vboxlayout->addWidget(toolbar, 0);
        mToolBar = toolbar;
    }
    vboxlayout->addWidget(mTextEdit);
    vboxlayout->addWidget(mActTextEdit);
    mActTextEdit->setVisible(false);
}

void SysModuleConfPanel::save(const QString &iRankStr)
{

    QString answer = TMessageBox::question(this,
                                           ttr("Modifying the configuration here is not recommended, this is a dangerous operation, sure to continue?"),
                                           "",
                                           ttr("Warning"),
                                           QStringList() << ttr("Save")+":Yes:Yes:Warn" << ttr("Cancel")+":Cancel:Cancel:Normal");
    if (answer != "Yes") {
        return;
    }
    TSqlUpdaterV2 updater;
    QVariantMap uData;
    QString table;
    QString where;
    QString errStr;
    QString productCategory = APP->productCategory();
    t::saving(this);
    try {
        if (mVersion.isEmpty()) {
            TSqlSelectorV2 selector;
            selector.setTable("sys_conf");
            selector.setField("version");
            QStringList path;
            path << "sys" << "product_version" << productCategory;
            selector.setWhere("path", QString("{%1}").arg(path.join(",")));
            QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_VALUE, QVariant::fromValue(selector));
            TDataResponse dataRes(data.toMap());
            if (dataRes.hasError()) {
                throw QString("update md5:get info fail(%1)").arg(dataRes.errText());
            } else {
                mVersion = dataRes.data().toString();
            }
        }

        if ("action" == myTypeIs()) {
            QString progStr = mTextEdit->toPlainText();
            QString attrStr = mActTextEdit->toPlainText();
            uData.insert(QString("attr_%1").arg(iRankStr),attrStr);
            uData.insert(QString("prog_%1").arg(iRankStr),progStr);
            table = TABLE_SYS_SETTING_ACTION;
            where = QString(" module_name = '%1' and action_name = '%2' ").arg(myModuleNameIs()).arg(myNameIs());
        } else if ("moduleConf" == myTypeIs()) {
            QString moduleConfStr = mTextEdit->toPlainText();
            table = TABLE_SYS_SETTING_MODULE;
            where = QString(" module_name = '%1' ").arg(myModuleNameIs());
            uData.insert(QString("conf_%1").arg(iRankStr),moduleConfStr);
        } else if ("language" == myTypeIs()) {
            QString languageStr = mTextEdit->toPlainText();
            table = TABLE_SYS_SETTING_MODULE;
            where = QString(" module_name = '%1' ").arg(myModuleNameIs());
            uData.insert(QString("lang_%1").arg(iRankStr),languageStr);
        } else if ("permission" == myTypeIs()) {
            QString permissionStr = mTextEdit->toPlainText();
            table = TABLE_SYS_SETTING_PERMISSION;
            where = QString(" permission_name = '%1' ").arg(myNameIs());
            uData.insert(QString("conf_%1").arg(iRankStr),permissionStr);
        } else if ("enum" == myTypeIs()) {
            QString enumStr = mTextEdit->toPlainText();
            table = TABLE_SYS_SETTING_ENUM;
            where = QString(" enum_name = '%1' ").arg(myNameIs());
            uData.insert(QString("conf_%1").arg(iRankStr),enumStr);
        }
        uData["md5"] = TDataParse::getVariantMd5(uData);
        updater.setData(uData);
        updater.setTable(table);
        updater.setWhere(where);
        updater.addWhere("version", mVersion);
        updater.addWhere("product_category", productCategory);

        QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_UPDATE_ROW, QVariant::fromValue(updater));
        TDataResponse dataRes(data.toMap());
        if (dataRes.hasError()) {
            errStr = QString("save fail(%1)").arg(dataRes.errText());
        } else {
            TSqlSelectorV2 selector;
            selector.setTable(table);
            selector.setWhere(where);
            selector.addWhere("version", mVersion);
            selector.addWhere("product_category", productCategory);

            QVariant data = doThreadWork(new TopClassSqlThread(this),TOPSQLTHREAD_SELECT_MAP,QVariant::fromValue(selector));
            TDataResponse dataRes(data.toMap());
            if (dataRes.hasError()) {
                errStr = QString("update md5:get info fail(%1)").arg(dataRes.errText());
            } else {
                QVariantMap dataMap = dataRes.data().toMap();
                dataMap.remove("id");
                dataMap.remove("md5");
                QVariantMap updateMap;
                updateMap["md5"] = TDataParse::getVariantMd5(dataMap);
                updater.clear();
                updater.setTable(table);
                updater.setWhere(where);
                updater.addWhere("version", mVersion);
                updater.addWhere("product_category", productCategory);
                updater.setData(updateMap);
                QVariant data = doThreadWork(new TopClassSqlThread(this),TOPSQLTHREAD_UPDATE_ROW,QVariant::fromValue(updater));
                TDataResponse dataRes(data.toMap());
                if(dataRes.hasError()){
                    errStr = QString("update md5 fail(%1)").arg(dataRes.errText());
                }
            }
        }
    } catch (const QString &err) {
        errStr = err;
    } catch (...) {
        errStr = ttr("Unknow Error!");
    }
    unloading();

    if (errStr.isEmpty()) {
        emit dataSaved(QVariant());
        setDataModified(false);
        alertOk(ttr("Data saved"));
    } else {
        TLOG_ERROR(errStr);
        alertError(ttr("Save failed"), errStr);
    }
}

void SysModuleConfPanel::cancel(const QString &iRankStr)
{
    QVariantMap sData;
    sData.insert("module_name", myModuleNameIs());
    sData.insert("name", myNameIs());
    sData.insert("type", myTypeIs());

    QVariant data = doThreadWork(new ModuleConfMgtThread(this), "LOAD_PANEL_DATA", QVariant::fromValue(sData));
    TDataResponse dataRes(data.toMap());
    if (!dataRes.hasError()) {
        QVariantMap dataMap = dataRes.data().toMap();
        bool isAction = false;
        if ("action" == dataMap.value("type")) {
            mActTextEdit->setVisible(true);
            isAction = true;
        } else {
            mActTextEdit->setVisible(false);
        }
        QStringList keys = dataMap.keys();
        for (QString key : keys) {
            if (key.contains(iRankStr)) {
                if (isAction) {
                    if (key.toLower().startsWith("attr")) {
                        mActTextEdit->setText(dataMap.value(key).toString());
                    } else if (key.toLower().startsWith("prog")) {
                        mTextEdit->setText(dataMap.value(key).toString());
                    }
                } else {
                    mTextEdit->setText(dataMap.value(key).toString());
                }

                if (!isAction) {
                    break;
                }
            }
        }
    }

    emit dataSaved(QVariant());
    setDataModified(false);
}
