﻿#include "sysuserthread.h"
#include <topcore/topclassabs.h>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>

SysUserThread::SysUserThread(QObject *iParent)
    : TopClassThreadAbs(iParent)
{

}

SysUserThread::~SysUserThread()
{

}

void SysUserThread::run()
{
    if (invokeName() == "LOAD_DATA") {
        loadData(invokeParameter().toInt());
    } else if (invokeName() == "SAVE_DATA") {
        saveData(invokeParameter().toMap());
    }
}

void SysUserThread::loadData(int iIdInt)
{
    TDataResponse dataRes;
    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin();
    try {
        TSqlSelectorV2 sqlSelector;
        sqlSelector.setTable("sys_user AS U LEFT JOIN pub_contacts AS C ON U.contcat_id = C.id");
        sqlSelector.setField("U.*, C.mail");
        sqlSelector.setWhere("U.id", QVariant(iIdInt));
        sqlSelector.setFieldFormat("product_category", "array");
        sqlSelector.setFieldFormat("attr_data", "json");
        QVariantMap dataMap = sqlQuery.selectMap(sqlSelector);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        sqlSelector.clear();
        sqlSelector.setTable("sys_role_map_user AS UR LEFT JOIN sys_role AS R ON UR.role_id = R.id");
        sqlSelector.fieldRef() << "R.id" << "R.name" << "R.sys_name" << "R.tags" << "R.description";
        sqlSelector.setWhere("UR.user_id", iIdInt);
        sqlSelector.setOrder("R.name", Qt::AscendingOrder);
        QVariantList roleList = sqlQuery.selectArrayMap(sqlSelector);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }
        dataMap.insert("ROLE_LIST", roleList);

        sqlQuery.commit();

        dataRes.setData(dataMap);
        setInvokeResult(dataRes.toVariantMap());
        return;
    } catch (const TError &err) {
        dataRes.setError(err);
    } catch (...) {
        dataRes.setErrText(ttr("Unknow Error!"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}

void SysUserThread::saveData(const QVariantMap &iDataMap)
{
    TDataResponse dataRes;

    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin();
    try {
        TSqlSelectorV2 sqlSelector;
        sqlSelector.setTable("sys_user");
        sqlSelector.setField("COUNT(1)");
        sqlSelector.addWhere("username", iDataMap.value("username"));
        sqlSelector.addWhere("id", iDataMap.value("id"), "!=");
        int count = sqlQuery.selectCount(sqlSelector);
        if (count > 0) {
            throw TError(ttr("'%1' already exists!").arg(iDataMap.value("username").toString()), "ERROR", "ALREADY_EXISTS");
        }

        // 更新pub_contacts表的mail
        QVariantMap contactMap;
        contactMap.insert("mail", iDataMap.value("mail"));
        QString username = iDataMap.value("username").toString();
        QString fullname = iDataMap.value("fullname").toString();
        QString name;
        if (!fullname.isEmpty() && (username != fullname)) {
            name = QString("%1(%2)").arg(fullname).arg(username);
        } else {
            name = username;
        }
        contactMap.insert("name", name);
        contactMap.insert("id", iDataMap.value("contcat_id").toInt());
        contactMap.insert("source", "company");
        contactMap.insert("status", iDataMap.value("status").toString() == "active" ? "active" : "inactive");
        TopClassAbs *parentModule = static_cast<TopClassAbs*>(this->parent());
        if (parentModule) {
            if (parentModule->isHookExists("handleContactData")) {
                contactMap = parentModule->callHooksQuick("handleContactData", QVariantList()<<contactMap<<iDataMap).toVariant().toMap();
            }
        }
        QStringList contactFields = contactMap.keys();
        contactFields.removeOne("id");
        TSqlInserterV2 sqlInserter;
        sqlInserter.setTable("pub_contacts");
        sqlInserter.setData(contactMap);
        sqlInserter.setField(contactFields);
        sqlInserter.setUniqueField("id");
        sqlInserter.setAutoIncrementField("id");

        QVariant contactId = sqlQuery.replaceRow(sqlInserter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        // 更新sys_commu_grp_detail
        TSqlUpdaterV2 sqlUpdater;
        sqlUpdater.setTable("sys_commu_grp_detail");
        sqlUpdater.setField(QStringList() << "address" << "name");
        QVariantMap commuGrpDetailDataMap;
        commuGrpDetailDataMap.insert("address", iDataMap.value("mail"));
        commuGrpDetailDataMap.insert("name", name);
        sqlUpdater.setData(commuGrpDetailDataMap);
        TSqlWhereCompsiteV2 commuGrpDetailWhere;
        commuGrpDetailWhere.append("src_type", "pub_contacts");
        commuGrpDetailWhere.append("src_uid", iDataMap.value("contcat_id").toString());
        commuGrpDetailWhere.setLogic(TSqlWhereCompsiteV2::Logic_And);
        sqlUpdater.setWhere(commuGrpDetailWhere);
        sqlQuery.updateRow(sqlUpdater);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        // 更新sys_user表
        QVariantMap dataMap = iDataMap;
        QStringList fieldLst = dataMap.keys();
        dataMap.insert("contcat_id", contactId);

        QStringList productCategory = dataMap.value("product_category").toStringList();
        // 只要产品分类列表中包含有*, 就只存*
        if (productCategory.contains("*")) {
            productCategory = (QStringList() << "*");
        }
        dataMap.insert("product_category", productCategory);

        if (iDataMap.value("id").toInt() == 0) {
            // 新建时插入默认密码
            QString password = iDataMap.value("password").toString();
            if (password.isEmpty()) {
                password = QStringLiteral("123456");
            }
            dataMap.insert("password", TDataParse::getVariantMd5(password));
        } else {
            fieldLst.removeOne("password");
        }
        fieldLst.removeOne("ROLE_LIST");
        fieldLst.removeOne("id");
        fieldLst.removeOne("mail");
        QVariantMap attrDataMap = dataMap.value("attr_data").toMap();
        if (attrDataMap.value("password_validity_control") == 0) {
            attrDataMap.insert("effective_date", QString(""));
            attrDataMap.insert("validity_period", QString(""));
            dataMap.insert("attr_data", attrDataMap);
        }
        qDebug().noquote().nospace()<<TDataParse::variant2JsonStr(dataMap);
        sqlInserter.clear();
        sqlInserter.setTable("sys_user");
        sqlInserter.setData(dataMap);
        sqlInserter.setField(fieldLst);
        sqlInserter.setUpdatePolicy("attr_data", "json_merge");
        sqlInserter.setUniqueField("id");
        sqlInserter.setAutoIncrementField("id");

        QVariant userId = sqlQuery.replaceRow(sqlInserter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        // 更新sys_role_map_user表
        TSqlDeleterV2 sqlDeleter;
        sqlDeleter.setTable("sys_role_map_user");
        sqlDeleter.setWhere("user_id", userId);
        sqlQuery.deleteRow(sqlDeleter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        QVariantList roleMapUserLst;
        for (QVariant role : iDataMap.value("ROLE_LIST").toList()) {
            QVariantMap roleMap = role.toMap();
            QVariantMap tempMap;
            tempMap.insert("user_id", userId.toString());
            tempMap.insert("role_id", roleMap.value("id").toString());
            roleMapUserLst << tempMap;
        }
        if (!roleMapUserLst.isEmpty()) {
            sqlQuery.batchInsert("sys_role_map_user",
                                 QStringList() << "user_id" << "role_id",
                                 roleMapUserLst);
            if (sqlQuery.lastError().isValid()) {
                throw sqlQuery.lastError();
            }
        }

        sqlQuery.commit();
        dataRes.setData(userId);
        setInvokeResult(dataRes.toVariantMap());
        return;
    } catch (const TError &err) {
        dataRes.setError(err);
    } catch (...) {
        dataRes.setErrText(ttr("Unknow Error!"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}
